/*
       B R O C C O L I  --  The Bro Client Communications Library

Copyright (C) 2004 Christian Kreibich <christian.kreibich(at)cl.cam.ac.uk>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifndef broccoli_type_h
#define broccoli_type_h

#include <bro_attrs.h>
#include <bro_object.h>

/* Internal types used to represent a Bro type.
 * Taken from Type.h.
 */
#define BRO_INTTYPE_INT            1
#define BRO_INTTYPE_UNSIGNED       2
#define BRO_INTTYPE_DOUBLE         3
#define BRO_INTTYPE_STRING         4
#define BRO_INTTYPE_IPADDR         5
#define BRO_INTTYPE_SUBNET         6
#define BRO_INTTYPE_OTHER          7
#define BRO_INTTYPE_ERROR          8

/* typedefs are in bro_types.h */

struct bro_type
{
  BroObject        object;

  /* Types can be chained together as a list here: */
  TAILQ_ENTRY(bro_type) type_list;

  char             tag;
  char             internal_tag;

  char             is_nbo;
  char             is_base_type;
  char             is_global_attrs_type;

  BroRecordType   *attrs_type;
};

struct bro_type_list
{
  BroObject        object;

  int              num_types;
  BroTypeTQ        types;
  BroType         *pure_type;
};

struct bro_type_decl
{
  /* Type declarations can be chained together as a list here: */
  TAILQ_ENTRY(bro_type_decl) td_list;

  BroAttrs        *attrs;
  BroType         *type;
  BroString        id;
};

struct bro_record_type
{
  BroType          type;

  BroTypeList     *base;
  int              num_fields;

  int              num_types;
  BroTypeDeclTQ    type_decls;
  
};

BroType         *__bro_type_new(void);
BroType         *__bro_type_new_of_type(int type);
void             __bro_type_init(BroType *type);
void             __bro_type_free(BroType *type);
int              __bro_type_read(BroType *type, BroConn *bc);
int              __bro_type_write(BroType *type, BroConn *bc);
int              __bro_type_clone(BroType *dst, BroType *src);


BroTypeList     *__bro_type_list_new(void);
void             __bro_type_list_init(BroTypeList *tl);
void             __bro_type_list_free(BroTypeList *tl);
int              __bro_type_list_read(BroTypeList *tl, BroConn *bc);
int              __bro_type_list_write(BroTypeList *tl, BroConn *bc);
int              __bro_type_list_clone(BroTypeList *dst, BroTypeList *src);


BroTypeDecl     *__bro_type_decl_new(void);
BroTypeDecl     *__bro_type_decl_copy(BroTypeDecl *td);
void             __bro_type_decl_free(BroTypeDecl *td);
int              __bro_type_decl_read(BroTypeDecl *td, BroConn *bc);
int              __bro_type_decl_write(BroTypeDecl *td, BroConn *bc);

BroRecordType   *__bro_record_type_new(void);
void             __bro_record_type_init(BroRecordType *rt);
void             __bro_record_type_free(BroRecordType *rt);
void             __bro_record_type_add_type(BroRecordType *rt, const char *field, BroType *type);

int              __bro_record_type_read(BroRecordType *rt, BroConn *bc);
int              __bro_record_type_write(BroRecordType *rt, BroConn *bc);
int              __bro_record_type_clone(BroRecordType *dst, BroRecordType *src);

#endif
